<?php

namespace Drupal\redirect_node\EventSubscriber;

use Drupal\Core\Routing\TrustedRedirectResponse;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\GetResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;
use Drupal\user\Entity\Role;

/**
 * Event handler to process redirection of a content type.
 *
 * @see
 *   https://www.thirdandgrove.com/redirecting-node-pages-drupal-8
 */
class RedirectRedirectNodes implements EventSubscriberInterface {

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    // This announces which events you want to subscribe to.
    // We only need the request event for this example.  Pass
    // this an array of method names.
    return([
      KernelEvents::REQUEST => [
        ['redirect'],
      ],
    ]);
  }

  /**
   * Redirect requests for `redirect` nodes to their destination url.
   *
   * @param \Symfony\Component\HttpKernel\Event\GetResponseEvent $event
   *   Event object of current event.
   */
  public function redirect(GetResponseEvent $event) {
    $request = $event->getRequest();

    // This is necessary because this also gets called on
    // node sub-tabs such as "edit", "revisions", etc.  This
    // prevents those pages from redirected.
    if ($request->attributes->get('_route') !== 'entity.node.canonical') {
      return;
    }

    // Only redirect a certain content type.
    $node = $request->attributes->get('node');
    if (empty($node) || $node->getType() !== 'redirect') {
      return;
    }

    $user = \Drupal::currentUser();

    $roles = Role::loadMultiple($user->getRoles());

    $isAdmin = array_reduce($roles, function ($carry, $item) {
      return $carry || $item->isAdmin();
    });

    // If user can edit the page, or is an admin, they should not be redirected.
    // If a user can't view the node, they should not be redirect and get a 403.
    if ($isAdmin || !$node->access() || $node->access("edit")) {
      return;
    }

    $uris = array_column($node->redirect_destination->getValue(), "uri");
    $destination = array_shift($uris);

    if (!empty($destination)) {
      // Create Redirect to the external URL.
      $response = new TrustedRedirectResponse($destination);

      // Don't Cache the redirect.
      $build = [
        '#cache' => [
          'contexts' => 'user',
        ],
      ];
      $response->addCacheableDependency($build, $user);

      // Execute the redic.
      $event->setResponse($response);

    }
  }

}
